import javax.swing.*;
import java.util.List;
import java.awt.*;
import java.awt.image.*;

/**
 * Class to show animation frames
 * @author Barb Ericson
 */
public class AnimationPanel extends JComponent
{
  /////////////// fields /////////////////////
  List pictureList = null;
  int startIndex = 0;
  int endIndex = 1;
  int framesPerSec = 30;
  JLabel frameLabel = null;
  
  ////////////// constructors /////////////////
  
  /**
   * Constructor that takes a list of pictures 
   * @param pictList the list of pictures
   */
  public AnimationPanel(List pictList, JLabel label)
  {
    this.pictureList = pictList; 
    Image image = ((Picture) this.pictureList.get(0)).getImage();
    BufferedImage bi = (BufferedImage) image;
    this.setSize(new Dimension(bi.getWidth(), 
                               bi.getHeight()));
    this.frameLabel = label;
  }
  
  /**
   * Constructor that takes a list of pictures and
   * the frames per second
   * @param pictList the list of pictures
   * @param frameRate the frames per second
   */
  public AnimationPanel(List pictList,JLabel label, int frameRate)
  {
    this(pictList,label);
    this.framesPerSec = frameRate;
  }
  
  ////////////// methods /////////////////////////
  
  /**
   * Method to add a picture
   * @param picture the picture to add
   */
  public void add(Picture picture)
  {
    pictureList.add(picture);
  }
  
  /**
   * Method to show just the next frame
   */
  public void showNext()
  {
    this.startIndex++;
    if (this.startIndex == pictureList.size())
      this.startIndex = 0;
    this.endIndex = this.startIndex + 1;
    this.frameLabel.setText("Frame Number " + startIndex);
    repaint();
  }
  
  /**
   * Method to show the previous frame
   */
  public void showPrev()
  {
    this.startIndex--;
    if (this.startIndex < 0)
      this.startIndex = pictureList.size() - 1;
    this.endIndex = this.startIndex + 1;
    this.frameLabel.setText("Frame Number " + startIndex);
    repaint();
  }
  
  /**
   * Method to show all the frames starting
   * with the first
   */
  public void showAll()
  {
    this.startIndex = 0;
    this.endIndex = this.pictureList.size();
    repaint();
  }
  
  /**
   * Method to paint the frames
   * @param g the graphics context to draw to
   */
  public void paintComponent(Graphics g)
  {
    Image image = null;
    int sleepTime = 1000 / framesPerSec;
    for (int i = startIndex; i < endIndex; i++)
    {
      image = ((Picture) pictureList.get(i)).getImage();
      g.drawImage(image,0,0,this);
      this.frameLabel.setText("Frame Number " + i);
      try {
        Thread.sleep(sleepTime);
      } catch (Exception ex) {
      }
    }
  }
    
}