                  import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.text.*;

/**
 * A class that represents a picture.  This class inherits from 
 * SimplePicture and allows the student to add functionality to
 * the Picture class.  
 * 
 * Copyright Georgia Institute of Technology 2004-2005
 * @author Barbara Ericson ericson@cc.gatech.edu
 */
public class Picture extends SimplePicture 
{
  ///////////////////// constructors //////////////////////////////////
  
  /**
   * Constructor that takes no arguments 
   */
  public Picture ()
  {
    /* not needed but use it to show students the implicit call to super()
     * child constructors always call a parent constructor 
     */
    super();  
  }
  
  /**
   * Constructor that takes a file name and creates the picture 
   * @param fileName the name of the file to create the picture from
   */
  public Picture(String fileName)
  {
    // let the parent class handle this fileName
    super(fileName);
  }
  
  /**
   * Constructor that takes the width and height
   * @param width the width of the desired picture
   * @param height the height of the desired picture
   */
  public Picture(int width, int height)
  {
    // let the parent class handle this width and height
    super(width,height);
  }
  
  /**
   * Constructor that takes a picture and creates a 
   * copy of that picture
   */
  public Picture(Picture copyPicture)
  {
    // let the parent class do the copy
    super(copyPicture);
  }
  
  ////////////////////// methods ///////////////////////////////////////
  
  /**
   * Method to return a string with information about this picture.
   * @return a string with information about the picture such as fileName,
   * height and width.
   */
  public String toString()
  {
    String output = "Picture, filename " + getFileName() + 
      " height " + getHeight() 
      + " width " + getWidth();
    return output;
    
  }
  
  public void ChangeRed (double change)
  {
    Pixel[] pixelArray = this.getPixels();
    Pixel pixelObj = null;
    int value =0;
    int index = 0;
    
    while (index < pixelArray.length)
    {
      pixelObj = pixelArray[index];
      value = pixelObj.getRed();
      value = (int)( value * change);
      pixelObj.setRed(value);
      index = index + 1;
    }
  }
  public void clearBlue ()
  {
    Pixel[] pixelArray = this.getPixels();
    Pixel pixelObj = null;
    for (int i = 0; i <pixelArray.length;i++){
    //loop through the array of pexels
   
      pixelObj = pixelArray[i];
      pixelObj.setBlue(0);
     
    }
  }
  public void clearBlueNested ()
  {
    Pixel[] pixelArray = this.getPixels();
    Pixel pixelObj = null;
    //loop through the columns (x direction)
   for (int x = 0; x < this.getWidth();x++)
     //loop through the rows (y direction)
     for (int y = 0; y < this.getHeight(); y++)
   
   {
   //get pixel at the x and y location
     pixelObj = this.getPixel (x,y);
     
     //set the pixel color to the lighter color
      pixelObj.setBlue(0);
     
    }
  }
  public void makeSunset()
  {
    Pixel[] pixelArray = this.getPixels();
    Pixel pixelObj = null;
    int green =0;
    int blue = 0;
    
//loop through all pixels
    for (int index =0; index < pixelArray.length;index++)
    {
      
   pixelObj = pixelArray[index];   
    blue = pixelObj.getBlue();
    pixelObj.setBlue((int)  (0.7 * blue));
  green = pixelObj.getGreen();
    pixelObj.setGreen((int)(0.7*green));
    }
  }
  public void negate()
  {
    Pixel[] pixelArray = this.getPixels();
      Pixel pixelObj = null;
      int redValue, blueValue, greenValue = 0;
      //loop through all the pixels 
      for (int i = 0; i < pixelArray.length;i++)
      {
        //get the current pixel 
        pixelObj = pixelArray[i];
        //get the values
        redValue = pixelObj.getRed();
        greenValue = pixelObj.getGreen();
        blueValue = pixelObj.getBlue();
        //set the pixel's color
        pixelObj.setColor(
                          new Color(255 - redValue,
                                    255 - greenValue,
                                    255 - blueValue));
        
  }
}
  public void grayscale()
  {
    Pixel[] pixelArray = this.getPixels();
    Pixel pixelObj = null;
    int intensity = 0;
    
    //loop through all the pixels
    for (int i = 0; i <pixelArray.length; i++)
    {
      //get the current pixel
      pixelObj = pixelArray[i];
      
      //compute the average intensity
      
      intensity =(int)
        (pixelObj.getRed() *.299+
          pixelObj.getGreen() * .587+
          pixelObj.getBlue()*.114);
         
         ///set the pixel color\
         pixelObj.setColor(new Color(intensity,intensity,intensity));
         }
  }
 public void lighten()
 {
   Pixel pixelObj = null;
   Color colorObj = null;
   //loop through the columns (x direction)
   for (int x = 0; x < this.getWidth();x++)
     //loop through the tows (y direction)
     for (int y = 0; y < this.getHeight(); y++)
   {
     //get pixel at teh x and y location
     pixelObj = this.getPixel (x,y);
    
     //get the current color 
     colorObj = pixelObj.getColor();
     
     //get a lighter color
     colorObj = colorObj.brighter();
     
     //set the pixel color to the lighter color
     pixelObj.setColor(colorObj);
   }
 }
public void copy(Picture sourcePicture)
{
  
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  for (int sourceX = 0, targetX=0;
         sourceX < sourcePicture.getWidth();
       sourceX++, targetX++)
  {
    //loop through the rows
    for (int sourceY=0, targetY=0;
         sourceY < sourcePicture.getHeight();
         sourceY++, targetY++)
    {
      //set the target pixel color to the source pixel color
      sourcePixel = sourcePicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
    }
  }
}
public void copyPosition(Picture sourcePicture)
{
  
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  for (int sourceX = 0, targetX=100;
         sourceX < sourcePicture.getWidth();
       sourceX++, targetX++)
  {
    //loop through the rows
    for (int sourceY=0, targetY=100;
         sourceY < sourcePicture.getHeight();
         sourceY++, targetY++)
    {
      //set the target pixel color to the source pixel color
      sourcePixel = sourcePicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
    }
  }
}
public void copyKatie()
{
  String sourceFile =
    FileChooser.getMediaPath ("KatieFancy.jpg");
  Picture sourcePicture = new Picture (sourceFile);
  this.copy (sourcePicture, 0,0, sourcePicture.getWidth(),
             sourcePicture.getHeight(),0,0);
}
public void copyKatiesFace()
{
  String sourceFile =
    FileChooser.getMediaPath ("KatieFancy.jpg");
  Picture sourcePicture = new Picture (sourceFile);
  
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  for (int sourceX =70, targetX=100;
         sourceX < 135; sourceX++, targetX++)
  {
    //loop through the rows
    for (int sourceY=3, targetY=100;
         sourceY < 80; sourceY++, targetY++)
    {
      //set the target pixel color to the source pixel color
      sourcePixel = sourcePicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
    }
  }
}
public void copyKatiesFeet()
{
  String sourceFile =
    FileChooser.getMediaPath ("KatieFancy.jpg");
  Picture sourcePicture = new Picture (sourceFile);
  
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  for (int sourceX =73, targetX=100;
         sourceX < 115; sourceX++, targetX++)
  {
    //loop through the rows
    for (int sourceY=331, targetY=100;
         sourceY < 343; sourceY++, targetY++)
    {
      //set the target pixel color to the source pixel color
      sourcePixel = sourcePicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
    }
  }
}
public void copy(Picture sourcePicture, int startX, int startY, 
                 int endX, int endY, int targetStartX, int targetStartY)
{
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  //loop through the x values
  for (int x = startX, tx = targetStartX;
       x < endX && x < sourcePicture.getWidth()&&
       tx<this.getWidth();
       x++, tx++)
  {
    //loop through the y values
    for (int y = startY, ty= targetStartY;
         y<endY && y<sourcePicture.getHeight()&&
         ty<this.getHeight();
         y++,ty++)
    {
      //copy the source color to the target color
      sourcePixel = sourcePicture.getPixel(x,y);
      targetPixel = this.getPixel (tx,ty);
      targetPixel.setColor(sourcePixel.getColor());
    }
  }
}
public void copyKatieLeftRotation()
{
  String sourceFile=
    FileChooser.getMediaPath ("KatieFancy.jpg");
  Picture sourcePicture = new Picture(sourceFile);
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  
 //loop through the columns
  for (int sourceX = 0;
       sourceX < sourcePicture.getWidth();
       sourceX++)
  {
    //loop through the rows
    for (int sourceY=0;
         sourceY < sourcePicture.getHeight();
         sourceY++)
  {
    //set the target pixel color to the source pixel color
    sourcePixel =
      sourcePicture.getPixel(sourceX, sourceY);
    targetPixel = this.getPixel(sourceY,
      sourcePicture.getWidth() - 1 - sourceX);
    targetPixel.setColor(sourcePixel.getColor());
    
  }
}
}
 
public void copyKatieRightRotation()
{
  String sourceFile=
    FileChooser.getMediaPath ("KatieFancy.jpg");
  Picture sourcePicture = new Picture(sourceFile);
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  int targetX, targetY;
 //loop through the columns
  for (int sourceX = 0;
       sourceX < sourcePicture.getWidth();
       sourceX++)
  {
    //loop through the rows
    for (int sourceY=0;
         sourceY < sourcePicture.getHeight();
         sourceY++)
  {
    //set the target pixel color to the source pixel color
    sourcePixel =
      sourcePicture.getPixel(sourceX, sourceY);
    targetX = sourcePicture.getHeight()-1-sourceY;
    targetY= sourceX;
    targetPixel = this.getPixel(targetX, targetY);
    targetPixel.setColor(sourcePixel.getColor());
    
  }
}
}
public void copyFLowerSmaller()
{
  Picture flowerPicture =
    new Picture(
                FileChooser.getMediaPath("passionFLower.jpg"));
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  
  //loop through the columns
  for (int sourceX = 0, targetX=0;
       sourceX<flowerPicture.getWidth();
       sourceX+=2, targetX++)
  {
    //loop through the rows
    for (int sourceY=0, targetY=0;
          sourceY <flowerPicture.getHeight();
          sourceY+=2, targetY++)
           {
      sourcePixel=
        flowerPicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
                           }
        }
  }
public void copySmaller(Picture sourcePicture)
{
  //Picture flowerPicture =
    //new Picture(
  //FileChooser.getMediaPath("passionFLower.jpg"));
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  
  //loop through the columns
  for (int sourceX = 0, targetX=0;
       sourceX<sourcePicture.getWidth();
       sourceX+=2, targetX++)
  {
    //loop through the rows
    for (int sourceY=0, targetY=0;
          sourceY <sourcePicture.getHeight();
          sourceY+=2, targetY++)
           {
      sourcePixel=
        sourcePicture.getPixel(sourceX,sourceY);
      targetPixel = this.getPixel(targetX, targetY);
      targetPixel.setColor(sourcePixel.getColor());
                           }
        }
  }
public void copyBigger(Picture sourcePicture)
{
  //Picture flowerPicture =
    //new Picture(
                //FileChooser.getMediaPath("flower1.jpg"));
  Pixel sourcePixel = null;
  Pixel targetPixel = null;
  
  //loop through the columns
  for (double sourceX = 0, targetX=0;
       sourceX<sourcePicture.getWidth();
       sourceX+=0.5, targetX++)
  {
    //loop through the rows
    for (double sourceY=0, targetY=0;
          sourceY <sourcePicture.getHeight();
          sourceY+=0.5, targetY++)
           {
      sourcePixel=
        sourcePicture.getPixel((int)sourceX,(int)sourceY);
      targetPixel = this.getPixel((int)targetX, (int)targetY);
      targetPixel.setColor(sourcePixel.getColor());
                           }
        }
  }
  public void mirrorVertical()
   {
    Pixel fromPixel = null;
    Pixel toPixel = null;
    int midpoint = this.getWidth()/2;
    for (int y = 0; y < this.getHeight();y++)
    {
      for(int x =0; x < midpoint; x++)
      {
        fromPixel = this.getPixel(x,y);
        toPixel = this.getPixel(this.getWidth () -1 -x,y);
        toPixel.setColor(fromPixel.getColor());
      }
    }
   }
  public void mirrorRight()
    {
    Pixel fromPixel = null;
    Pixel toPixel = null;
    int midpoint = this.getWidth()/2;
    for (int y = 0; y < this.getHeight();y++)
    {
      for(int x =0; x < midpoint; x++)
      {
       toPixel = this.getPixel(x,y);
        fromPixel = this.getPixel(this.getWidth () -1 -x,y);
        toPixel.setColor(fromPixel.getColor());
      }
    }
   }
  
  public void mirrorHorizontal()
    {
    Pixel fromPixel = null;
    Pixel toPixel = null;
    int MidPoint = this.getHeight()/2;
    for (int y = 0; y < MidPoint;y++)
    {
      for(int x =0; x < this.getWidth(); x++)
      {
       fromPixel = this.getPixel(x,y);
        toPixel = this.getPixel(x,this.getHeight () -1 -y);
        toPixel.setColor(fromPixel.getColor());
      }
    }
   }
  public void createCollage()
  {
    Picture source1Picture = 
      new Picture(FileChooser.getMediaPath ("flower1.jpg"));
    Picture source2Picture = 
      new Picture(FileChooser.getMediaPath ("flower2.jpg"));
    int targetBottomY = this.getHeight()- 5;

    
    // copy source1Picture to 0,0, targetBottomY - height
  this.copy(source1Picture,0,0,
            source1Picture.getWidth(),
            source1Picture.getHeight(),
            0, targetBottomY - 
            source1Picture.getHeight());
  
  //copy source2Picture to 100, targetBottomY - height
    this.copy(source2Picture,0,0,
              source2Picture.getWidth(),
              source2Picture.getHeight(),
              100, targetBottomY - 
              source2Picture.getHeight());
   
    //negate the sourcePicture
    source1Picture.negate();
      this.copy(source1Picture,0,0,
            source1Picture.getWidth(),
            source1Picture.getHeight(),
           200, targetBottomY - 
            source1Picture.getHeight());
      //
      this.copy(source2Picture,0,0,
              source2Picture.getWidth(),
              source2Picture.getHeight(),
              300, targetBottomY - 
              source2Picture.getHeight());
   
      this.copy(source1Picture,0,0,
            source1Picture.getWidth(),
            source1Picture.getHeight(),
           400, targetBottomY - 
            source1Picture.getHeight());
      
      
              }

public void MyCollagePicture()
  {
    Picture source1Picture = 
      new Picture(FileChooser.getMediaPath ("redMotorcycle.jpg"));
    Picture source2Picture = 
      new Picture(FileChooser.getMediaPath ("robot.jpg"));
    int targetBottomY = this.getHeight()- 5;

    
    // copy source1Picture to 0,0, targetBottomY - height
    source2Picture.grayscale();
 this.copy (source2Picture,0,0,
            source2Picture.getWidth(),
            source2Picture.getHeight(),
           50,40);
 
  
  source1Picture.clearBlue();
 this.copy (source1Picture,0,0,
            source1Picture.getWidth(),
            source1Picture.getHeight(),
            100,40);

 //make sunset()
   
  source2Picture.makeSunset();
 this.copy (source2Picture,0,0,
            source2Picture.getWidth(),
            source2Picture.getHeight(),
            250,40);
 
 //make mirrorRight()
 this.copy (source1Picture,0,0,
            source1Picture.getWidth(),
            source1Picture.getHeight(),
            400,40);
 
 
 
   
}

 
    
 
   public void removeRedEye (int startX, int startY, int endX, int endY, Color newColor)
   {
     Pixel pixelObj = null;
     
     //loop through the pixels in the rectangle defined by the
     //startX, startY, and endX and endY
     for (int x = startX; x < endX; x++)
     {
       for (int y = startY; y<endY; y++)
       {
         //get the currrent pixel
         pixelObj = getPixel(x,y);
         //if the color is near red then change it
         if(pixelObj.colorDistance(Color.red)<167)
           pixelObj.setColor(newColor);
       }
     }
   }
  public void edgeDetection(double limit)
    
   {
    Pixel bottomPixel = null;
    Pixel topPixel = null;
     
     double distance;
     //loop through the pixels in the rectangle defined by the
     //startX, startY, and endX and endY
     for (int x = 0; x <this.getWidth(); x++)
     {
       for (int y = 0; y<this.getHeight()-1; y++)
       {
         topPixel =this.getPixel(x,y);
         bottomPixel = this.getPixel(x,y+1);
         
         Color bottomColor=bottomPixel.getColor();
         distance =topPixel.colorDistance(bottomColor);
         
         if (distance <= limit)
         {
           topPixel.setColor(Color.WHITE);
         }
            else
             {
               topPixel.setColor(Color.BLACK);
            }
       }
     }
  }

 public void edgeDetection2(double limit)
    
   {
    Pixel rightPixel = null;
    Pixel leftPixel = null;
     
     double distance;
     //loop through the pixels in the rectangle defined by the
     //startX, startY, and endX and endY
     for (int x = 0; x <this.getWidth() -1; x++)
     {
       for (int y = 0; y<this.getHeight(); y++)
       {
         leftPixel =this.getPixel(x,y);
         rightPixel = this.getPixel(x+y,1);
         
         Color rightColor=rightPixel.getColor();
         distance =leftPixel.colorDistance(rightColor);
         
         if (distance <= limit)
         {
           leftPixel.setColor(Color.WHITE);
         }
            else
             {
               leftPixel.setColor(Color.BLACK);
            }
       }
     }
  }

 public void sepiaTint()
{
  Pixel pixelObj=null;
  double redValue = 0;
  double greenValue = 0;
  double blueValue =0;
  
  //first change the current picture to grayscale
  this.grayscale();
  
  //loop through the pixels
  for (int x = 0; x<this.getWidth(); x++)
  {
    for (int y = 0; y<this.getHeight(); y++)
    {
      // get the current pixel and color values
      pixelObj=this.getPixel(x,y);
      redValue=pixelObj.getRed();
      greenValue=pixelObj.getGreen();
        blueValue = pixelObj.getBlue();
        //tint the shadows darker
        if(redValue <60)
        {
          redValue=redValue *0.9;
          greenValue=greenValue*0.9;
          blueValue=greenValue*0.9;
        }
        
        //tint the midtones a light brown by reducing the blue
        else if(redValue<190)
        {
          blueValue=blueValue*0.8;
        }
        
        //tint the highlights a light yellow
        //by reducing the blue
        else
        {
          blueValue = blueValue*0.9;
        }
        
        //set the colors
        pixelObj.setRed((int) redValue);
        pixelObj.setGreen((int)greenValue);
        pixelObj.setBlue((int)blueValue);
    }
  }
}
 public void swapBackground(Picture oldBackground, Picture newBackground)
 {
   Pixel currPixel = null;
     Pixel oldPixel = null;
   Pixel newPixel = null;
   
   //loop through the columns
   for (int x = 0; x<this.getWidth();x++)
   {
     
     //loop through the rows
     for (int y=0; y<this.getHeight();y++)
     {
       
       //get the current pixel and old background pixel
       currPixel = this.getPixel(x,y);
       oldPixel = oldBackground.getPixel(x,y);
       
       /* if the distance between the current pixel color
        * *and the old background pixel color is less than the 15
        * then swap in the new background pixel
        * */
       if (currPixel.colorDistance(oldPixel.getColor())<15.0)
       {
         newPixel = newBackground.getPixel(x,y);
         currPixel.setColor(newPixel.getColor());
       }
     }
   }
   }
 public void chromakey(Picture newBackground)
 {
   Pixel currPixel= null;
   Pixel newPixel = null;
   
   //loop through the columns
   for (int x = 0; x<this.getWidth();x++)
   {
     
     //loop through the rows
     for (int y=0; y<this.getHeight();y++)
     {
       
       //get the current pixel and old background pixel
       currPixel = this.getPixel(x,y);
       if (currPixel.getRed()+currPixel.getBlue()< currPixel.getGreen())
       {
         newPixel = newBackground.getPixel(x,y);
         currPixel.setColor(newPixel.getColor());
       }
     }
   }
 } 
 public void posterize()
   {
   int result=0;
  Pixel pixelObj=null;
  double redValue = 0;
  double greenValue = 0;
  double blueValue =0;
  
  //first change the current picture to grayscale
  //this.grayscale();
  
  //loop through the pixels
  for (int x = 0; x<this.getWidth(); x++)
  {
    for (int y = 0; y<this.getHeight(); y++)
    {
      // get the current pixel and color values
      pixelObj=this.getPixel(x,y);
      redValue=pixelObj.getRed();
      greenValue=pixelObj.getGreen();
        blueValue = pixelObj.getBlue();
        
        //tint the shadows darker
        if(redValue>=0 &&  redValue <= 63)
        {
          redValue=31;
        } 
      else if(redValue>=64 && redValue<=127)
        {
      redValue=95;
      }
      else if(redValue>=128 && redValue<=191)
      {
       redValue=159;
      }
      else if(redValue>=192 && redValue<=255)
      {
        redValue=223;
      }
         
        if(greenValue>=0 &&  greenValue <= 63)
        {
          greenValue=31;
        } 
      else if(greenValue>=64 && greenValue<=127)
        {
      greenValue=95;
      }
      else if(greenValue>=128 && greenValue<=191)
      {
       greenValue=159;
      }
      else if(greenValue>=192 && greenValue<=255)
      {
        greenValue=223;
      } 
       if(blueValue>=0 &&  blueValue <= 63)
        {
          blueValue=31;
        } 
      else if(blueValue>=64 && blueValue<=127)
        {
      blueValue=95;
      }
      else if(blueValue>=128 && blueValue<=191)
      {
       blueValue=159;
      }
      else if(blueValue>=192 && blueValue<=255)
      {
        blueValue=223;
      }
        //tint the highlights a light yellow
        //by reducing the blue
        else
        {
          blueValue = blueValue*0.9;
        }
        
        //set the colors
        pixelObj.setRed((int) redValue);
        pixelObj.setGreen((int)greenValue);
        pixelObj.setBlue((int)blueValue);
    }
  }
 }
}

// end of class Picture, put all new methods before this
 
